package eu.etaxonomy.taxeditor.prototype2.view.propertysheetsupport;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.beans.BeansObservables;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.TreeItem;

import eu.etaxonomy.taxeditor.prototype2.model.PropertySheetNode;
import eu.etaxonomy.taxeditor.prototype2.view.PropertySheetViewer;

/**
 * 
 * Borrowed from tomson <tom.schindl@bestsolution.at>'s GenericObservableMapCellLabelProvider
 * 
 * @author p.ciardelli
 *
 */
public class PropertySheetValueLabelProvider extends ColumnLabelProvider {
	
	private DataBindingContext bindingContext;

	public PropertySheetValueLabelProvider(PropertySheetViewer viewer) {
		this.bindingContext = viewer.getBindingContext();
	}

	public Image getImage(Object element) {
		return null;
	}

	public String getText(Object element) {
		return (String) ((PropertySheetNode) element).getPropertyValue();
	}
			
	public void update(ViewerCell cell) {
	
		Object element = cell.getElement();
		cell.setText(getText(element));
		Image image = getImage(element);
		cell.setImage(image);
		cell.setBackground(getBackground(element));
		cell.setForeground(getForeground(element));
		cell.setFont(getFont(element));

		// if node element is observing a value, bind it to its TreeItem
		IObservableValue observeNodeValue = ((PropertySheetNode) element).getObserveValue();
		if (observeNodeValue != null) {
			
	        Assert.isNotNull(bindingContext,
                    "Label provider: PropertySheetViewer's binding context must be explicitly set.");
		
	        TreeItemColumnBean itemColumn = new TreeItemColumnBean((TreeItem) cell.getItem(),1);
			IObservableValue observeCellValue = BeansObservables.observeValue(itemColumn, "text");
			bindingContext.bindValue(observeCellValue, observeNodeValue, null, null);
		}
	}
	
	/**
	 * TreeItem returns column texts by index, i.e. getText(1) - this is a wrapper which 
	 *  allows BeansObservables to observe value with getText / setText
	 **/
	class TreeItemColumnBean {

		TreeItem item;
		int index;
		
		TreeItemColumnBean(TreeItem item, int index) {
			this.item = item;
			this.index = index;
		}

		public String getText() {
			return item.getText(index);
		}

		public void setText(String string) {
			item.setText(index, string);
		}
	}
}