// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.images;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.images.ImagesController;
import eu.etaxonomy.taxeditor.editor.images.TaxonImagesEditor;

/**
 * @author p.ciardelli
 * @created 30.03.2009
 * @version 1.0
 */
public class ImagePropertySource implements IPropertySource {
	private static final Logger logger = Logger
			.getLogger(ImagePropertySource.class);
	
	private URL url;

	private Taxon taxon;

    // Property unique keys
	public static final String P_ID_URL = "url";

    // Property display keys
	// Note: for an explanation of the sorting prefixes ("04:"), 
	//  @see eu.etaxonomy.taxeditor.propertysheet.CustomSortPropertySheetEntry
	public static final String P_URL = "00:URL";
	
	/**
	 * @param url
	 */
	public ImagePropertySource(Taxon taxon, URL url) {
		this.taxon = taxon;
		this.url = url;
		
		addDescriptor(P_ID_URL);
	}

	protected Vector<PropertyDescriptor> descriptors = new Vector<PropertyDescriptor>();
	
	protected void addDescriptor(String id) {
		if (id.equals(P_ID_URL)) {
			descriptors.addElement(
					new TextPropertyDescriptor(P_ID_URL, P_URL));
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
	 */
	public Object getEditableValue() {
		return this;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		return (IPropertyDescriptor[]) descriptors.toArray(
                new IPropertyDescriptor[descriptors.size()]);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(Object id) {
        if (id.equals(P_ID_URL)) {
			return url.toString();
        }
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
	 */
	public boolean isPropertySet(Object id) {
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
	 */
	public void resetPropertyValue(Object id) {}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object id, Object value) {
		if (id.equals(P_ID_URL)) {
			
			try {
				
				// TODO put this in an operation to allow undo
				URL newUrl = new URL((String) value);
				
				ImagesController.removeTaxonImageUrl(taxon, url);
				ImagesController.addTaxonImageUrl(taxon, newUrl);
				
				url = newUrl;
			} catch (MalformedURLException e) {
				// Do nothing if URL is malformed
			} catch (IOException e) {
				// Do nothing if URL causes an IO Exception
			}
		}
	}
}
