/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.descriptiontree;

import org.apache.log4j.Logger;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.IDetailsPage;
import org.eclipse.ui.forms.IFormPart;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import eu.etaxonomy.cdm.model.common.ReferencedEntityBase;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.dialogs.filteredSelection.FilteredReferenceSelectionDialog;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author p.ciardelli
 *
 */
abstract public class AbstractDescriptionDetailsPage implements IDetailsPage, IPropertyChangeListener {	
	private static final Logger logger = Logger.getLogger(AbstractDescriptionDetailsPage.class);

	private static final String NO_CITATION_MSG = "No citation specified.";

	private static final int DEFAULT_WIDTH = 300;

	protected IManagedForm mform;
	protected Object input;
	protected Section section;
	protected Composite client;
	protected FormToolkit toolkit;
	private Text citationText;
	private Label detailLabel;
	private Text detailText;
	private boolean hasCitationControls = false;
		
	private boolean citationListenersEnabled;

	private Composite citationComposite;

	private Button btnEdit;

	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.IDetailsPage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	public void createContents(Composite parent) {
						
		TableWrapLayout layout = new TableWrapLayout();
		layout.topMargin = 5;
		layout.leftMargin = 5;
		layout.rightMargin = 2;
		layout.bottomMargin = 2;
		parent.setLayout(layout);
		TableWrapData td2 = new TableWrapData(TableWrapData.FILL_GRAB, TableWrapData.FILL_GRAB);
		parent.setLayoutData(td2);		
		
		toolkit = mform.getToolkit();
		section = toolkit.createSection(parent, Section.DESCRIPTION|Section.TITLE_BAR);
		section.marginWidth = 10;
		TableWrapData td = new TableWrapData(TableWrapData.FILL_GRAB, TableWrapData.FILL_GRAB);
		td.maxWidth = DEFAULT_WIDTH;
		td.grabHorizontal = true;
		section.setLayoutData(td);
		
		client = toolkit.createComposite(section, SWT.NONE);
		GridLayout glayout = new GridLayout();
		glayout.marginWidth = glayout.marginHeight = 0;
		client.setLayout(glayout);
		TableWrapData tdclient = new TableWrapData(TableWrapData.FILL_GRAB, TableWrapData.FILL_GRAB);
		client.setLayoutData(tdclient);
		
		section.setClient(client);
		
		PreferencesUtil.getPreferenceStore().addPropertyChangeListener(this);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		if (PreferencesUtil.PREFERRED_TERMS_CHANGE.equals(event.getProperty())) {
			handleChangedPrefs(event);
		}
	}
	
	/**
	 * Sample usage:
	 * <pre>	
	 * if (LanguageMenuPreferences.class.equals(event.getNewValue())) {
	 *     populateLanguages();
	 * }
	 * </pre>
	 * 
	 * @param event
	 */
	protected void handleChangedPrefs(PropertyChangeEvent event) {
		// do nothing
	}

	protected void createCitationControls() {
		
		hasCitationControls  = true;
		
		// Citation
		citationComposite = toolkit.createComposite(client);
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		citationComposite.setLayout(layout);
		GridData gd = new GridData(GridData.GRAB_HORIZONTAL
                | GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_FILL
                | GridData.VERTICAL_ALIGN_CENTER | GridData.FILL_HORIZONTAL);
        citationComposite.setLayoutData(gd);
        
		citationText = toolkit.createText(citationComposite, "", SWT.BORDER);
		citationText.setEditable(false);
		citationText.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
                | GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_FILL
                | GridData.VERTICAL_ALIGN_BEGINNING | GridData.FILL_HORIZONTAL));
		
		btnEdit = toolkit.createButton(citationComposite, "Edit ...", SWT.PUSH);
		btnEdit.addMouseListener(new MouseAdapter() {
			public void mouseUp(org.eclipse.swt.events.MouseEvent e) {
				getReferenceFromDialog();
			}
		});
		btnEdit.setLayoutData(new GridData(GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_END
                | GridData.VERTICAL_ALIGN_BEGINNING ));
		
		Button btnRemove = toolkit.createButton(citationComposite, "Remove", SWT.PUSH);
		btnRemove.addMouseListener(new MouseAdapter() {
			public void mouseUp(org.eclipse.swt.events.MouseEvent e) {				
				if (((ReferencedEntityBase) input).getCitation() == null) {
					return;
				}
				setCitation(null);
				((ReferencedEntityBase) input).setCitation(null);
				fireStateChanged();
			}
		});
		btnRemove.setLayoutData(new GridData(GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_END
                | GridData.VERTICAL_ALIGN_BEGINNING ));
		
		// Citation details
		final Composite detailComposite = toolkit.createComposite(client);
		detailComposite.setLayout(new RowLayout());		
		detailComposite.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
                | GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_FILL
                | GridData.VERTICAL_ALIGN_CENTER));
        
        detailLabel = toolkit.createLabel(detailComposite, "Citation detail: ");
        
        detailText = toolkit.createText(detailComposite, "", SWT.BORDER);
        
        detailText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				if (citationListenersEnabled) {
					((ReferencedEntityBase) input).setCitationMicroReference(detailText.getText());
					fireStateChanged();
				}
			}
        	
        });
	}	
	
	/**
	 * 
	 */
	protected void getReferenceFromDialog() {

//		ReferenceBase reference = ((ReferencedEntityBase) input).getCitation();
		ReferenceBase reference = FilteredReferenceSelectionDialog.selectReference
									(client.getShell(), ((ReferencedEntityBase) input).getCitation());
		
		if (reference != null) {
			setCitation(reference);
			((ReferencedEntityBase) input).setCitation(reference);
			fireStateChanged();
		}
	}

	/**
	 * @param detail
	 */
	private void setCitationDetail(String detail) {
		detailText.setText(detail == null ? "" : detail);
	}

	/**
	 * @param citation
	 */
	private void setCitation(ReferenceBase citation) {
		String text = citation == null ? NO_CITATION_MSG : citation.getTitleCache();
		if (text == null || text.equals("")) {
			text = "Citation without title cache";
		}
		text += "  ";
				
		citationText.setText(text);

		citationText.getParent().getParent().layout();
		citationText.getParent().layout();		
	}

	/**
	 * Alerts editor that dirty state has changed.
	 */
	protected void fireStateChanged() {
		mform.dirtyStateChanged();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.IFormPart#commit(boolean)
	 */
	public void commit(boolean onSave) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.IFormPart#dispose()
	 */
	public void dispose() {
		PreferencesUtil.getPreferenceStore().removePropertyChangeListener(this);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.IFormPart#initialize(org.eclipse.ui.forms.IManagedForm)
	 */
	public void initialize(IManagedForm form) {
		this.mform = form;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.IFormPart#isDirty()
	 */
	public boolean isDirty() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.IFormPart#isStale()
	 */
	public boolean isStale() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.IFormPart#refresh()
	 */
	public void refresh() {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.IFormPart#setFocus()
	 */
	public void setFocus() {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.IFormPart#setFormInput(java.lang.Object)
	 */
	public boolean setFormInput(Object input) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.IPartSelectionListener#selectionChanged(org.eclipse.ui.forms.IFormPart, org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IFormPart part, ISelection selection) {	
		IStructuredSelection ssel = (IStructuredSelection)selection;
		if (ssel.size()==1) {
			input = ssel.getFirstElement();
		} else {
			input = null;
		}
		
		if (hasCitationControls && input instanceof ReferencedEntityBase) {
			setCitationControlListenersEnabled(false);
			setCitation(((ReferencedEntityBase)input).getCitation());
			setCitationDetail(((ReferencedEntityBase)input).getCitationMicroReference());
			setCitationControlListenersEnabled(true);
		}
	}

	/**
	 * @param enabled
	 */
	private void setCitationControlListenersEnabled(boolean enabled) {
		citationListenersEnabled = enabled;
	}

	protected Label createLabel(String string, Composite composite, Composite parent) {
		Label label = new Label(composite, SWT.WRAP);
        label.setText(string);
        GridData data = new GridData(GridData.GRAB_HORIZONTAL
                | GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_FILL
                | GridData.VERTICAL_ALIGN_CENTER);
        label.setLayoutData(data);
        label.setFont(parent.getFont());
        return label;
	}
}
