/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.descriptiontree;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.IFormPart;
import org.eclipse.ui.forms.events.HyperlinkAdapter;
import org.eclipse.ui.forms.events.HyperlinkEvent;
import org.eclipse.ui.forms.widgets.Hyperlink;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.TermLanguageComparator;
import eu.etaxonomy.cdm.model.description.CommonTaxonName;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.taxeditor.model.DescriptionHelper;
import eu.etaxonomy.taxeditor.preference.LanguageMenuPreferences;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * If this is a common name whose language has not been chosen yet, the last explicitly
 * chosen language will be displayed in the language dropdown.
 * 
 * @author p.ciardelli
 *
 */
public class CommonNameDetailsPage extends AbstractDescriptionDetailsPage implements ModifyListener, SelectionListener {
	
	private static final Logger logger = Logger.getLogger(CommonNameDetailsPage.class);
	
	private Text text;
	private Combo languagesCombo;
	
	private static final int VISIBLE_ITEMS = 20;
	
	/**
	 * Language selected last time the dialog was used.
	 */
	private static Language lastLanguage;	
	
	private List<Language> languages;

	private boolean selectionListenersEnabled;

	private Language selectedLanguage;

	private Composite msgComposite;
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
	 */
	public void modifyText(ModifyEvent e) {
		if (selectionListenersEnabled) {
			DescriptionHelper.setCache((DescriptionElementBase) input, text.getText());
			fireStateChanged();
		}
	}	
	
	/**
	 * @param language
	 */
	private void modifyLanguage(Language language) {
		if (selectionListenersEnabled) {
			((CommonTaxonName) input).setLanguage(language);
			fireStateChanged();		
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetDefaultSelected(SelectionEvent e) {}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	
	public void widgetSelected(SelectionEvent e) {
		Language language = languages.get(languagesCombo.getSelectionIndex());
		modifyLanguage(language);				
		setLastSelectedLanguage(language);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.descriptionnew.AbstractDescriptionDetailsPage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	public void createContents(Composite parent) {
		super.createContents(parent);
		
		section.setText("Common Name Details"); //$NON-NLS-1$
		
		createLabel("Language:", client, parent);
						
        // create combo
        languagesCombo = new Combo(client, SWT.BORDER | SWT.READ_ONLY);
        languagesCombo.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
                | GridData.HORIZONTAL_ALIGN_FILL));
        languagesCombo.setVisibleItemCount(VISIBLE_ITEMS);
		languagesCombo.addSelectionListener(this);
                
        createLabel("Common name:", client, parent);
        
		text = new Text(client, SWT.BORDER | SWT.MULTI);
		GridData gd = new GridData();	
		gd.horizontalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		text.setLayoutData(gd);
        languagesCombo.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
                | GridData.HORIZONTAL_ALIGN_FILL));		
		
		text.addModifyListener(this);
        
        populateLanguages();
        
        createCitationControls();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.descriptiontree.AbstractDescriptionDetailsPage#handleChangedPrefs(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	protected void handleChangedPrefs(PropertyChangeEvent event) {
		if (LanguageMenuPreferences.class.equals(event.getNewValue())) {
			populateLanguages();
		}
	}

	/**
	 * @param language
	 */
	private void setLastSelectedLanguage(Language language) {
		lastLanguage = language;
	}

	private void populateLanguages() {
		
		languagesCombo.removeAll();
		
		languages = new ArrayList<Language>();
		languages.add(0, null);
		languagesCombo.add("--- No language selected ---");
		
		int i = 0;		
		int index = 0;
		List<Language> languagesTemp = PreferencesUtil.getPreferredLanguages();
		Collections.sort(languagesTemp, new TermLanguageComparator<Language>());
		for (Language language : languagesTemp) {
			String description = language.getDescription(); 
			if (description == null) {
				logger.warn("No description for " + language + " - not added to language menu");
				continue;
			}
			languagesCombo.add(description);
			languages.add(language);

			i++;
			if (selectedLanguage != null) {
				if (selectedLanguage.equals(language)) {
			 		index = i;
				}
			} else {
				// TODO isn't it better to put the last language at the top of the menu, not automatically select?
//					if (lastLanguage != null && lastLanguage.equals(language)) {
//						index = i;  
//					}
			}
		}
		
		// Check whether language is in menu
		if (msgComposite != null) {
			msgComposite.dispose();
			msgComposite = null;
		}
		if (selectedLanguage != null && index == 0) {
			createLanguageNotInMenu(selectedLanguage);
		}
		
		languagesCombo.select(index);		
	}
	
	/**
	 * @param language
	 */
	private void createLanguageNotInMenu(Language language) {
		
		msgComposite = toolkit.createComposite(client);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		msgComposite.setLayout(layout);
		msgComposite.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
                | GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_FILL
                | GridData.VERTICAL_ALIGN_CENTER));
		
		String langText = language.getDescription() == null ? language.getLabel() : language.getDescription();
		langText += " is not one of your preferred languages. ";
		Label label = toolkit.createLabel(msgComposite, langText, SWT.WRAP);
		label.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
                | GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_FILL
                | GridData.VERTICAL_ALIGN_CENTER));
		
		Hyperlink linkEdit = toolkit.createHyperlink(msgComposite, "Edit preferences", SWT.WRAP);
		linkEdit.addHyperlinkListener(new HyperlinkAdapter() {
			/* (non-Javadoc)
			 * @see org.eclipse.ui.forms.events.HyperlinkAdapter#linkActivated(org.eclipse.ui.forms.events.HyperlinkEvent)
			 */
			@Override
			public void linkActivated(HyperlinkEvent e) {
				org.eclipse.ui.dialogs.PreferencesUtil.createPreferenceDialogOn(client.getShell(), 
						LanguageMenuPreferences.ID, null, null).open();
			}
		});
		
		msgComposite.moveBelow(languagesCombo);
		
		client.pack(true);
		client.layout();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.descriptionnew.AbstractDescriptionDetailsPage#selectionChanged(org.eclipse.ui.forms.IFormPart, org.eclipse.jface.viewers.ISelection)
	 */
	@Override
	public void selectionChanged(IFormPart part, ISelection selection) {
		super.selectionChanged(part, selection);
		
		setSelectionListenersEnabled(false);
		
		text.setText(((CommonTaxonName) input).getName());
		selectedLanguage = (((CommonTaxonName) input).getLanguage());
		populateLanguages();
		
		setSelectionListenersEnabled(true);
	}

	/**
	 * @param enabled
	 */
	private void setSelectionListenersEnabled(boolean enabled) {
		this.selectionListenersEnabled = enabled;
	}
}