/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.descriptiontree;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.forms.DetailsPart;
import org.eclipse.ui.forms.IDetailsPage;
import org.eclipse.ui.forms.IDetailsPageProvider;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.MasterDetailsBlock;
import org.eclipse.ui.forms.SectionPart;
import org.eclipse.ui.forms.editor.FormPage;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;

import eu.etaxonomy.cdm.model.description.CommonTaxonName;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;
import eu.etaxonomy.taxeditor.model.ImagesHelper;

/**
 * @author p.ciardelli
 *
 */
public class DescriptionMasterDetailsBlock extends MasterDetailsBlock {
	private static final Logger logger = Logger
			.getLogger(DescriptionMasterDetailsBlock.class);

	private FormPage page;

	private TreeViewer descriptionViewer;

	private IDescriptiveInformationProvider descriptiveInformationProvider = new DescriptiveInformationProvider();
	
	public DescriptionMasterDetailsBlock(FormPage page) {
		this.page = page; 
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.MasterDetailsBlock#createMasterPart(org.eclipse.ui.forms.IManagedForm, org.eclipse.swt.widgets.Composite)
	 */
	protected void createMasterPart(final IManagedForm managedForm,
			Composite parent) {
		FormToolkit toolkit = managedForm.getToolkit();
		Section section = toolkit.createSection(parent, Section.NO_TITLE);
		section.marginWidth = 10;
		section.marginHeight = 5;
		Composite client = toolkit.createComposite(section, SWT.WRAP);
		GridLayout layout = new GridLayout();
		layout.marginWidth = 2;
		layout.marginHeight = 2;
		client.setLayout(layout);
		
		Button btnExpand = toolkit.createButton(client, "Show all descriptive data", SWT.PUSH);
		btnExpand.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
		btnExpand.addMouseListener(new MouseAdapter() {
			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.MouseAdapter#mouseUp(org.eclipse.swt.events.MouseEvent)
			 */
			@Override
			public void mouseUp(MouseEvent e) {
				descriptionViewer.expandAll();
			}
		});
		
		Tree tree = toolkit.createTree(client, SWT.NULL);
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.heightHint = 20;
		gd.widthHint = 100;
		tree.setLayoutData(gd);
		toolkit.paintBordersFor(client);
		section.setClient(client);
		final SectionPart spart = new SectionPart(section);
		managedForm.addPart(spart);
				
		descriptionViewer = new TreeViewer(tree);
		descriptionViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			private ISelection lastSelection;

			public void selectionChanged(SelectionChangedEvent event) {
				if (lastSelection != null) {
					// Update label of last selection
					Object item;
					TreePath[] paths = ((TreeSelection) lastSelection).getPaths();
					if (paths.length > 0) {
						item = ((TreeSelection) lastSelection).getPaths()[0].getLastSegment();
						descriptionViewer.update(item, null);
					}
				}
 				managedForm.fireSelectionChanged(spart, event.getSelection());
 				lastSelection = event.getSelection();
			}
		});

		descriptionViewer.setAutoExpandLevel(2); // Show descriptions and features
		descriptionViewer.setContentProvider(new DescriptionContentProvider(descriptiveInformationProvider));
		descriptionViewer.setLabelProvider(new DescriptionLabelProvider());
		descriptionViewer.setInput(page.getEditor().getEditorInput());
		descriptionViewer.setComparator(new ViewerComparator() {
			/* (non-Javadoc)
			 * @see org.eclipse.jface.viewers.ViewerComparator#compare(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
			 */
			@Override
			public int compare(Viewer viewer, Object e1, Object e2) {
				if (e1 instanceof ImageFile && e2 instanceof ImageFile) {
					return 0;
				}
				return super.compare(viewer, e1, e2);
			}
		});
				
		// Set tree as page's selection provider
		page.getSite().setSelectionProvider(descriptionViewer);
		
		// Add context menu to tree
		MenuManager menuMgr = new MenuManager();
		page.getSite().registerContextMenu(page.getId(), menuMgr, descriptionViewer);
		Menu menu = menuMgr.createContextMenu(descriptionViewer.getControl());
		descriptionViewer.getControl().setMenu(menu);	
	}
	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.MasterDetailsBlock#createToolBarActions(org.eclipse.ui.forms.IManagedForm)
	 */
	protected void createToolBarActions(IManagedForm managedForm) {
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.MasterDetailsBlock#registerPages(org.eclipse.ui.forms.DetailsPart)
	 */
	protected void registerPages(DetailsPart detailsPart) {
		detailsPart.registerPage(ImageFile.class, new ImageDetailsPage());
		detailsPart.setPageProvider(new PageProvider());
	}

	/**
	 * @param objectAffectedByOperation
	 */
	public synchronized void refreshTree(Object revealObject) {		
		descriptionViewer.refresh(true);
		
		Object[] segments = null;
		if (revealObject instanceof DescriptionElementBase) {
			DescriptionElementBase element = (DescriptionElementBase) revealObject;
			TaxonDescriptionFeature tdf = descriptiveInformationProvider.getDescriptionFeature(element);
			
			segments = new Object[3];
			segments[0] = element.getInDescription();
			segments[1] = tdf;
			segments[2] = element;
		}
		if (revealObject instanceof ImageFile) {
			ImageFile image = (ImageFile) revealObject;
			DescriptionElementBase element = ImagesHelper.findImageElement(image, getTaxon());
			
			segments = new Object[3];
			segments[0] = element.getInDescription();
			segments[1] = element;
			segments[2] = image;			
		}
		if (revealObject instanceof TaxonDescription) {
			TaxonDescription description = (TaxonDescription) revealObject;
			segments = new Object[] {(TaxonDescription) revealObject};	
		}
		if (segments != null) {
			TreePath treePath = new TreePath(segments);
			descriptionViewer.setSelection(new TreeSelection(treePath), true);			
		}
	}
	
	/**
	 * @return
	 */
	private Taxon getTaxon() {
		IEditorPart editor = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
		if (editor != null && editor.getEditorInput() instanceof TaxonEditorInput) {
			return ((TaxonEditorInput) editor.getEditorInput()).getTaxon();
		}
		return null;
	}

	/**
	 * @author p.ciardelli
	 */
	class PageProvider implements IDetailsPageProvider {

		private final static String IMAGE_GALLERY_ELEMENT = "isImageGalleryElement";
		private final static String IMAGE_GALLERY_DESCRIPTION = "isImageGalleryDescription";

		/* (non-Javadoc)
		 * @see org.eclipse.ui.forms.IDetailsPageProvider#getPage(java.lang.Object)
		 */
		public IDetailsPage getPage(Object key) {
			
			// Descriptions
			if (IMAGE_GALLERY_DESCRIPTION.equals(key)) {
				return new LabelDetailsPage("Image Gallery Details", "Edit image gallery's label:");
			}
			if (TaxonDescription.class.equals(key)) {
				return new LabelDetailsPage("Description Details", "Edit description's label:");
			}
			
			// Features
			if (Feature.DISTRIBUTION().equals(key)) {
				return new DistributionMapDetailsPage();
			}
			
			// Description elements
			if (IMAGE_GALLERY_ELEMENT.equals(key)) {
				return new LabelDetailsPage("Image List Details", "Edit ordered image list's label:");
			}
			if (CommonTaxonName.class.equals(key)) {
				return new CommonNameDetailsPage();
			}
			if (Distribution.class.equals(key)) {
				return new DistributionDetailsPage();
			}
			if (TextData.class.equals(key)) {
				return new TextDataDetailsPage();
			}			
			return null;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.ui.forms.IDetailsPageProvider#getPageKey(java.lang.Object)
		 */
		public Object getPageKey(Object object) {
			
			// Descriptions
			if (object instanceof TaxonDescription) {
				if (((TaxonDescription) object).isImageGallery()) {
					return IMAGE_GALLERY_DESCRIPTION;
				}
			}
			
			// Features
			if (object instanceof TaxonDescriptionFeature) {
				return ((TaxonDescriptionFeature) object).getFeature();
			}
			
			// Description elements
			if (object instanceof DescriptionElementBase) {
				TaxonDescription description = (TaxonDescription) ((DescriptionElementBase) object).getInDescription();
				if (description != null && description.isImageGallery()) {
					return IMAGE_GALLERY_ELEMENT;
				}
			}
			return object.getClass();
		}
	}
}