/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.descriptiontree;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.IFormPart;

import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.taxeditor.model.DescriptionHelper;

/**
 * Edit the label for a TaxonDescription, or for a DescriptionElementBase whose cache is used
 * as a label, e.g. a DescriptionElementBase which contains an image list.
 * 
 * @author p.ciardelli
 *
 */
public class LabelDetailsPage extends AbstractDescriptionDetailsPage implements ModifyListener{
	
	private Text text;
	private String sectionHeader;
	private String description;
	
	public LabelDetailsPage(String sectionHeader, String description) {
		this.sectionHeader = sectionHeader; 
		this.description = description;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
	 */
	public void modifyText(ModifyEvent e) {
		saveLabel();
		fireStateChanged();
	}	
	
	/**
	 * 
	 */
	private void saveLabel() {
		if (input instanceof TaxonDescription) {
			((TaxonDescription) input).setTitleCache(text.getText());
		}
		if (input instanceof DescriptionElementBase) {
			DescriptionHelper.setCache((DescriptionElementBase) input, text.getText());
		}
	}
	
	private String getLabel() {
		if (input instanceof TaxonDescription) {
			return ((TaxonDescription) input).getTitleCache();
		}
		if (input instanceof DescriptionElementBase) {
			return DescriptionHelper.getCache((DescriptionElementBase) input);
		}
		return "";
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.descriptionnew.AbstractDescriptionDetailsPage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	public void createContents(Composite parent) {
		super.createContents(parent);

		section.setText(sectionHeader); //$NON-NLS-1$
		createLabel(description, client, parent);
		text = new Text(client, SWT.BORDER | SWT.MULTI);
		
		GridData gd = new GridData();	
		gd.horizontalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;

		gd.heightHint = 100;
		text.setLayoutData(gd);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.descriptionnew.AbstractDescriptionDetailsPage#selectionChanged(org.eclipse.ui.forms.IFormPart, org.eclipse.jface.viewers.ISelection)
	 */
	@Override
	public void selectionChanged(IFormPart part, ISelection selection) {
		super.selectionChanged(part, selection);
		text.removeModifyListener(this);
		text.setText(getLabel());
		text.addModifyListener(this);
	}
}