/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name;

import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.views.properties.IPropertySource;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.operations.ChangeConceptRelationshipTypeOperation;
import eu.etaxonomy.taxeditor.operations.ChangeConceptToSynonymOperation;
import eu.etaxonomy.taxeditor.propertysheet.name.ConceptPropertySource;

/**
 * @author p.ciardelli
 * @created 26.01.2009
 * @version 1.0
 */
public class ConceptComposite extends NameComposite<Taxon> {
	
	private TaxonRelationshipType relationshipType;
	
		
	private ConceptComposite(TaxonNameEditor editor, Composite parent,
			Taxon relatedTaxon, TaxonRelationshipType relationshipType, 
			boolean acceptedTaxonIsFromTaxon) {
		super(editor, parent, relatedTaxon);

		// FIXME the acceptedTaxonIsFromTaxon is never used and 
		// I can't remember what is was used for in the first place
		
		this.relationshipType = relationshipType;
		showRelationshipType();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.NameComposite#initializeComposite()
	 */
	@Override
	protected void initializeComposite() {
		
		setIsDraggable(true);
		setIcon(CONCEPT_ICON);
		setFont(getViewerFont());
		setIndent(CONCEPT_INDENT);
		
		showSec();
				
		initTextViewer(getTaxonBase());
	}
	
	private void showRelationshipType() {
		setNonEditableInfo(relationshipType.getLabel());
	}

	private void showSec() {
		if (getTaxonBase() == null) {
			return;
		}
		
		if (getTaxonBase().getSec() == null) {
			setNonEditableInfo("sec. ???");
		} else {
			setNonEditableInfo("sec. " + getTaxonBase().getSec().getTitleCache());
		}
	}

	public static ConceptComposite getNewInstance(TaxonNameEditor editor, Composite parent, TaxonRelationship relationship){

		TaxonRelationshipType relationshipType = relationship.getType();
		
		if (relationship.getToTaxon().equals(editor.getTaxon())) {
			return new ConceptComposite (editor, parent, 
					relationship.getFromTaxon(), relationshipType, false);
		} else {
			return new ConceptComposite (editor, parent, 
					relationship.getToTaxon(), relationshipType, true);
		} 
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.GroupedComposite#getViewerFont()
	 */
	@Override
	protected Font getViewerFont() {
		return CONCEPT_FONT;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.IHasPropertySource#getPropertySource()
	 */
	
	public IPropertySource getPropertySource() {
		return new ConceptPropertySource(getTaxonBase());
	}

	public Taxon getRelatedTaxon() {
		return getTaxonBase();
	}
	
	public TaxonNameBase<?, ?> getName() {
		return getTaxonBase().getName();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.NameComposite#setName(eu.etaxonomy.cdm.model.name.TaxonNameBase)
	 */
	@Override
	public void setName(TaxonNameBase name) {
		getTaxonBase().setName(name);
	}
	
	protected void calculateErrors() {
		super.calculateErrors();
		
		textViewer.setShowSecError(getTaxonBase());
	}
	
	public Object getData () {
		return getRelatedTaxon();
	}
	
	public boolean setParent(Composite parent) {
		boolean doSetParent = false;
		
		if (parent instanceof MisappliedGroupComposite) {
			IUndoableOperation operation = new ChangeConceptRelationshipTypeOperation("change to misapplication", 
					editor.getUndoContext(), getEditorTaxon(), getTaxonBase(), TaxonRelationshipType.MISAPPLIED_NAME_FOR(), editor); //$NON-NLS-1$
			EditorUtil.executeOperation(operation);
			
			doSetParent = true;
		}
		
		if (parent instanceof HomotypicalGroupComposite) {
			HomotypicalGroup homotypicalGroup = 
				((HomotypicalGroupComposite)parent).getGroup();
			IUndoableOperation operation = new ChangeConceptToSynonymOperation
					("change concept to synonym", editor.getUndoContext(), getEditorTaxon(), getTaxonBase(), homotypicalGroup, editor);
			EditorUtil.executeOperation(operation);
		
			doSetParent = true;
		}
		
		return doSetParent;
	}
}






