/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet;

import java.util.Set;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.DialogCellEditor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.ui.views.properties.PropertyDescriptor;

import eu.etaxonomy.cdm.model.common.AnnotatableEntity;

/**
 * Property sheet descriptor that pops up a dialog window
 *  returning a set.
 *  
 * @author p.ciardelli
 * @created 19.05.2008
 * @version 1.0
 */
abstract public class AnnotationsPropertyDescriptor extends PropertyDescriptor {
	
	private AnnotatableEntity entity;
	
	public AnnotationsPropertyDescriptor(Object id, String displayName, AnnotatableEntity entity) {
		super(id, displayName);
		this.entity = entity;
	}

    public CellEditor createPropertyEditor(Composite parent) {
        CellEditor editor = new DialogCellEditor(parent) {

        	protected Dialog dialog;
        	
			@Override
			protected Object openDialogBox(
					Control cellEditorWindow) {
		        dialog = new AnnotationsDialog(cellEditorWindow.getShell(), entity);
		        Object value = ((AnnotationsDialog) dialog).open();
				if (value instanceof Set) {
					
					Set<?> annotations = (Set<?>) value;
					saveAnnotations(annotations);
					return new AnnotationPropertySource(entity);
				}
				return null;
			}
        	
        };
        if (getValidator() != null) {
			editor.setValidator(getValidator());
		}
        return editor;
    }
        
    abstract protected void saveAnnotations(Set<?> set);	
}
