/**
 * 
 */
package eu.etaxonomy.taxeditor.propertysheet;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.common.MarkerType;

/**
 * @author p.ciardelli
 *
 */
public class SelectMarkerTypeDialog extends InputDialog {

	private Set<MarkerType> markerTypes;
	private ListViewer textViewer;
	String message = "Choose a type for the new marker";
	private Button okButton;
	private Set<MarkerType> selectedMarkerTypes;

	/**
	 * @param parentShell
	 * @param markerTypes 
	 */
	protected SelectMarkerTypeDialog(Shell shell, Set<MarkerType> markerTypes) {
		super(shell, "Create a new marker", 
				"Choose a type for the new marker", null, null);
		
		this.markerTypes = markerTypes;
	}
	
    protected Control createDialogArea(Composite parent) {
        // create composite
		Composite composite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.marginHeight = convertVerticalDLUsToPixels(IDialogConstants.VERTICAL_MARGIN);
		layout.marginWidth = convertHorizontalDLUsToPixels(IDialogConstants.HORIZONTAL_MARGIN);
		layout.verticalSpacing = convertVerticalDLUsToPixels(IDialogConstants.VERTICAL_SPACING);
		layout.horizontalSpacing = convertHorizontalDLUsToPixels(IDialogConstants.HORIZONTAL_SPACING);
		composite.setLayout(layout);
		composite.setLayoutData(new GridData(GridData.FILL_BOTH));
		applyDialogFont(composite);
        // create message
        if (message != null) {
            Label label = new Label(composite, SWT.WRAP);
            label.setText(message);
            GridData data = new GridData(GridData.GRAB_HORIZONTAL
                    | GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_FILL
                    | GridData.VERTICAL_ALIGN_CENTER);
            data.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
            label.setLayoutData(data);
            label.setFont(parent.getFont());
        }
        textViewer = new ListViewer(composite);
        textViewer.getList().setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
                | GridData.HORIZONTAL_ALIGN_FILL));
        textViewer.setContentProvider(new ContentProvider());
        textViewer.setLabelProvider(new LabelProvider() {
        	/* (non-Javadoc)
        	 * @see org.eclipse.jface.viewers.LabelProvider#getText(java.lang.Object)
        	 */
        	@Override
        	public String getText(Object element) {
        		return ((MarkerType) element).getLabel();
        	}
        });
        textViewer.setInput(markerTypes);
        
        textViewer.addSelectionChangedListener(new ISelectionChangedListener () {
			public void selectionChanged(SelectionChangedEvent event) {
				setOkEnabled(true);
			}
		});
        textViewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				setSelectedMarkerTypes();
				close();
			}
        });

        applyDialogFont(composite);
        return composite;
    }
    

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.InputDialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent) {
        // create OK and Cancel buttons by default
        okButton = createButton(parent, IDialogConstants.OK_ID,
                IDialogConstants.OK_LABEL, true);
        okButton.addMouseListener(new MouseAdapter() {
        	/* (non-Javadoc)
        	 * @see org.eclipse.swt.events.MouseAdapter#mouseUp(org.eclipse.swt.events.MouseEvent)
        	 */
        	@Override
        	public void mouseUp(MouseEvent e) {
        		close();
        	}
        });
        createButton(parent, IDialogConstants.CANCEL_ID,
                IDialogConstants.CANCEL_LABEL, false);
        setOkEnabled(false);
    }
    
	/**
	 * @param enabled
	 */
	protected void setOkEnabled(boolean enabled) {
		okButton.setEnabled(enabled);
	}
	    
    /* (non-Javadoc)
     * @see org.eclipse.jface.dialogs.Dialog#close()
     */
    @Override
    public boolean close() {
    	setSelectedMarkerTypes();
    	return super.close();
    }
    
	/**
	 * @return 
	 * 
	 */
	private void setSelectedMarkerTypes() {
		selectedMarkerTypes = new HashSet<MarkerType>();
		
		IStructuredSelection selection = (IStructuredSelection) textViewer.getSelection();
		Iterator iterator = selection.iterator();
		while (iterator.hasNext()) {
			selectedMarkerTypes.add((MarkerType) iterator.next());
		}
	}
    
    class ContentProvider implements IStructuredContentProvider {

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object inputElement) {
			return markerTypes.toArray();
		}

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {}

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}
    	
    }

	/**
	 * @return 
	 * 
	 */
	public Set<MarkerType> getMarkerTypes() {
		return selectedMarkerTypes;
	}
}
