/**
 * 
 */
package eu.etaxonomy.taxeditor.propertysheet.authors;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Vector;

import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.IPage;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.PropertySheet;
import org.eclipse.ui.views.properties.PropertySheetPage;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.taxeditor.propertysheet.ICdmBasePropertySource;
import eu.etaxonomy.taxeditor.propertysheet.reference.ReferencePropertySource;

/**
 * @author p.ciardelli
 *
 */
public class AuthorTeamPropertySource implements ICdmBasePropertySource {

	private Team team;
	
	public static final String P_ID_TITLE = "P_ID_TITLE";
	public static final String P_ID_NOM_TITLE = "P_ID_NOM_TITLE";
	public static final String P_ID_PROTECT_CACHE = "P_ID_PROTECT_CACHE";
	public static final String P_ID_AUTHORS = "P_ID_AUTHORS";

	public static final String P_TITLE = "Editable title cache";
	public static final String P_NOM_TITLE = "Nomenclatural title";
	public static final String P_PROTECT_CACHE = "Protect title cache?";
	public static final String P_AUTHORS = "Authors";

	private static final int CACHE_NOT_PROTECTED = 0;
	private static final int CACHE_PROTECTED = 1;
	
	public AuthorTeamPropertySource(Team team) {
		this.team = team;
		
		initDescriptors();
	}
	
	/**
	 * 
	 */
	private void initDescriptors() {
		addDescriptor(P_ID_TITLE);
		addDescriptor(P_ID_PROTECT_CACHE);
		addDescriptor(P_ID_NOM_TITLE);
		addDescriptor(P_ID_AUTHORS);
	}

	protected Vector<PropertyDescriptor> descriptors = new Vector<PropertyDescriptor>();
	
	/**
	 * @param field
	 */
	private void addDescriptor(String id) {
		
		// Editable cache
		if (id.equals(P_ID_TITLE)) {
			descriptors.addElement(
					new TextPropertyDescriptor(P_ID_TITLE, P_TITLE));
		}
		
		// Editable cache
		if (id.equals(P_ID_NOM_TITLE)) {
			descriptors.addElement(
					new TextPropertyDescriptor(P_ID_NOM_TITLE, P_NOM_TITLE));
		}
		
		// Protect cache?
		if (id.equals(P_ID_PROTECT_CACHE)) {
			descriptors.addElement(
					new ComboBoxPropertyDescriptor(P_ID_PROTECT_CACHE, P_PROTECT_CACHE, 
							new String[] {"no", "yes"}));			
		}
		
		boolean isProtectedCache = team.isProtectedTitleCache();
		
		// Reference search
		if (id.equals(P_ID_AUTHORS)) {
			descriptors.addElement(
					isProtectedCache ?
							new PropertyDescriptor(P_ID_AUTHORS, P_AUTHORS) :
							new TeamAuthorsListDescriptor(P_ID_AUTHORS, P_AUTHORS, team));
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		return (IPropertyDescriptor[]) descriptors.toArray(
                new IPropertyDescriptor[descriptors.size()]);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
	 */
	public Object getEditableValue() {
		// TODO Auto-generated method stub
		return null;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(Object id) {
		
		if (id.equals(P_ID_TITLE)) {
			return team.getTitleCache();
		}

		if (id.equals(P_ID_NOM_TITLE)) {
			return team.getNomenclaturalTitle();
		}
		
		if (id.equals(P_ID_PROTECT_CACHE)) {
			if (team.isProtectedTitleCache()) {
				return CACHE_PROTECTED;
			} else {
				return CACHE_NOT_PROTECTED;
			}
		}
		
		if (id.equals(P_ID_AUTHORS)) {
			return new TeamAuthorsListPropertySource(team.getTeamMembers());
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
	 */
	public boolean isPropertySet(Object id) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
	 */
	public void resetPropertyValue(Object id) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object id, Object value) {
		
		if (id.equals(P_ID_TITLE)) {
			team.setTitleCache((String) value);
		}

		if (id.equals(P_ID_NOM_TITLE)) {
			team.setNomenclaturalTitle((String) value);
		}
		
		// Protect cache? 
		if (id.equals(P_ID_PROTECT_CACHE)) {
			if (((Integer) value).intValue() == CACHE_PROTECTED) {
				team.setProtectedTitleCache(true);
			} else {
				team.setProtectedTitleCache(false);
			}
//			IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
//			if (window != null) {
//				IWorkbenchPart part = window.getActivePage().getActivePart();
//				if (part instanceof PropertySheet) {
//					IPage page = ((PropertySheet) part).getCurrentPage();
//					if (page instanceof PropertySheetPage) {
//						((PropertySheetPage) page).refresh();
//					}
//				}
//				IWorkbenchPart part2 = window.getActivePage().getActivePart();
//			}
		}
	}
}
