// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation.handler;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.model.taxon.TaxonomicTree;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.operations.DeleteTaxonNodeOperation;
import eu.etaxonomy.taxeditor.operations.DeleteTaxonomicTreeOperation;

/**
 * @author n.hoffmann
 * @created 06.04.2009
 * @version 1.0
 */
public class DeleteTaxonNodeHandler extends AbstractHandler implements IHandler{
	private static final Logger logger = Logger
			.getLogger(DeleteTaxonNodeHandler.class);

	public Object execute(ExecutionEvent event) throws ExecutionException {
		
		// TODO handle delete on multiple nodes
		
		TreeSelection selection = (TreeSelection) HandlerUtil.getCurrentSelection(event);
		Object treeOrNode = selection.getFirstElement();
		
		IUndoableOperation operation = null;
		try {
			if(treeOrNode instanceof TaxonomicTree){
				operation = new DeleteTaxonomicTreeOperation(event.getCommand().getName(), 
						NavigationUtil.getUndoContext(), (TaxonomicTree) treeOrNode);
			}else if(treeOrNode instanceof TaxonNode){
				
				operation = new DeleteTaxonNodeOperation(event.getCommand().getName(), 
						NavigationUtil.getUndoContext(), (TaxonNode) treeOrNode);
				
			}
			IStatus status = NavigationUtil.executeOperation(operation);
			
			// Close any open taxon node editors
			if (status == Status.OK_STATUS && treeOrNode instanceof TaxonNode) {
				IWorkbenchPage activePage = HandlerUtil.getActiveWorkbenchWindow(event).getActivePage();
				for (IEditorReference ref : activePage.getEditorReferences()) {
					try {
						IEditorInput input = ref.getEditorInput(); 
						if (input instanceof TaxonEditorInput) {
							TaxonNode node = ((TaxonEditorInput) input).getTaxonNode();
							if (treeOrNode.equals(node)) {
								activePage.closeEditor(ref.getEditor(false), false);
							}
						}
					} catch (PartInitException e) {
						continue;
					}
				}
			}
		} catch (NotDefinedException e) {
			logger.warn("Command name not set");
		}
		return null;
	}
}
